@@ -37,8 +37,8 @@
  * file under either the MPL or the LGPL.
  */
 
-#ifndef _AGENT_H
-#define _AGENT_H
+#ifndef __LIBNICE_AGENT_H__
+#define __LIBNICE_AGENT_H__
 
 /**
  * SECTION:agent
@@ -54,13 +54,22 @@
  *
  * A #NiceAgent must always be used with a #GMainLoop running the #GMainContext
  * passed into nice_agent_new() (or nice_agent_new_reliable()). Without the
- * #GMainContext being iterated, the agent’s timers will not fire and, if
- * nice_agent_attach_recv() is used, packets will not be received.
+ * #GMainContext being iterated, the agent’s timers will not fire, etc.
  *
  * Streams and their components are referenced by integer IDs (with respect to a
  * given #NiceAgent). These IDs are guaranteed to be positive (i.e. non-zero)
  * for valid streams/components.
  *
+ * To complete the ICE connectivity checks, the user must either register
+ * an I/O callback (with nice_agent_attach_recv()) or call nice_agent_recv_messages()
+ * in a loop on a dedicated thread.
+ * Technically, #NiceAgent does not poll the streams on its own, since
+ * user data could arrive at any time; to receive STUN packets
+ * required for establishing ICE connectivity, it is backpiggying
+ * on the facility chosen by the user. #NiceAgent will handle all STUN
+ * packets internally; they're never actually passed to the I/O callback
+ * or returned from nice_agent_recv_messages() and related functions.
+ *
  * Each stream can receive data in one of two ways: using
  * nice_agent_attach_recv() or nice_agent_recv_messages() (and the derived
  * #NiceInputStream and #NiceIOStream classes accessible using
@@ -98,7 +107,9 @@
  *   stream_id = nice_agent_add_stream (agent, 1);
  *   nice_agent_gather_candidates (agent, stream_id);
  *
- *   // Attach to the component to receive the data
+ *   // Attach I/O callback the component to ensure that:
+ *   // 1) agent gets its STUN packets (not delivered to cb_nice_recv)
+ *   // 2) you get your own data
  *   nice_agent_attach_recv (agent, stream_id, 1, NULL,
  *                          cb_nice_recv, NULL);
  *
@@ -563,9 +574,9 @@ nice_agent_gather_candidates (
  * nice_agent_set_remote_credentials:
  * @agent: The #NiceAgent Object
  * @stream_id: The ID of the stream
- * @ufrag: NULL-terminated string containing an ICE username fragment
+ * @ufrag: nul-terminated string containing an ICE username fragment
  *    (length must be between 22 and 256 chars)
- * @pwd: NULL-terminated string containing an ICE password
+ * @pwd: nul-terminated string containing an ICE password
  *    (length must be between 4 and 256 chars)
  *
  * Sets the remote credentials for stream @stream_id.
@@ -592,6 +603,33 @@ nice_agent_set_remote_credentials (
   const gchar *ufrag, const gchar *pwd);
 
 
+/**
+ * nice_agent_set_local_credentials:
+ * @agent: The #NiceAgent Object
+ * @stream_id: The ID of the stream
+ * @ufrag: nul-terminated string containing an ICE username fragment
+ *    (length must be between 22 and 256 chars)
+ * @pwd: nul-terminated string containing an ICE password
+ *    (length must be between 4 and 256 chars)
+ *
+ * Sets the local credentials for stream @stream_id.
+ *
+ <note>
+   <para>
+     This is only effective before ICE negotiation has started.
+   </para>
+ </note>
+ *
+ * Since 0.1.11
+ * Returns: %TRUE on success, %FALSE on error.
+ */
+gboolean
+nice_agent_set_local_credentials (
+  NiceAgent *agent,
+  guint stream_id,
+  const gchar *ufrag,
+  const gchar *pwd);
+
 
 /**
  * nice_agent_get_local_credentials:
@@ -854,11 +892,14 @@ nice_agent_restart_stream (
  * @component_id: The ID of the component
  * @ctx: The Glib Mainloop Context to use for listening on the component
  * @func: The callback function to be called when data is received on
- * the stream's component
+ * the stream's component (will not be called for STUN messages that
+ * should be handled by #NiceAgent itself)
  * @data: user data associated with the callback
  *
  * Attaches the stream's component's sockets to the Glib Mainloop Context in
- * order to be notified whenever data becomes available for a component.
+ * order to be notified whenever data becomes available for a component,
+ * and to enable #NiceAgent to receive STUN messages (during the
+ * establishment of ICE connectivity).
  *
  * This must not be used in combination with nice_agent_recv_messages() (or
  * #NiceIOStream or #NiceInputStream) on the same stream/component pair.
@@ -928,6 +969,11 @@ nice_agent_recv (
  * and written into @messages, the stream is closed by the other end or by
  * calling nice_agent_remove_stream(), or @cancellable is cancelled.
  *
+ * Any STUN packets received will not be added to @messages; instead,
+ * they'll be passed for processing to #NiceAgent itself. Since #NiceAgent
+ * does not poll for messages on its own, it's therefore essential to keep
+ * calling this function for ICE connection establishment to work.
+ *
  * In the non-error case, in reliable mode, this will block until all buffers in
  * all @n_messages have been filled with received data (i.e. @messages is
  * treated as a large, flat array of buffers). In non-reliable mode, it will
@@ -1024,6 +1070,11 @@ nice_agent_recv_nonblocking (
  * into @messages until it would block; in non-reliable mode, it will receive
  * messages until it would block.
  *
+ * Any STUN packets received will not be added to @messages; instead,
+ * they'll be passed for processing to #NiceAgent itself. Since #NiceAgent
+ * does not poll for messages on its own, it's therefore essential to keep
+ * calling this function for ICE connection establishment to work.
+ *
  * As this function is non-blocking, @cancellable is included only for parity
  * with nice_agent_recv_messages(). If @cancellable is cancelled before this
  * function is called, a %G_IO_ERROR_CANCELLED error will be returned
@@ -1202,8 +1253,12 @@ void nice_agent_set_software (
  * @stream_id: The ID of the stream to change
  * @name: The new name of the stream or %NULL
  *
- * This function will assign a unique name to a stream.
- * This is only useful when parsing and generating an SDP of the candidates.
+ * This function will assign a media type to a stream. The only values
+ * that can be used to produce a valid SDP are: "audio", "video",
+ * "text", "application", "image" and "message".
+ *
+ * This is only useful when parsing and generating an SDP of the
+ * candidates.
  *
  * <para>See also: nice_agent_generate_local_sdp()</para>
  * <para>See also: nice_agent_parse_remote_sdp()</para>
@@ -1529,5 +1584,4 @@ nice_agent_get_component_state (NiceAgent *agent,
 
 G_END_DECLS
 
-#endif /* _AGENT_H */
-
+#endif /* __LIBNICE_AGENT_H__ */
