@@ -59,11 +59,33 @@ static volatile unsigned int n_components_destroyed = 0;
 #include "discovery.h"
 #include "agent-priv.h"
 
+G_DEFINE_TYPE (NiceComponent, nice_component, G_TYPE_OBJECT);
+
+typedef enum {
+  PROP_ID = 1,
+  PROP_AGENT,
+  PROP_STREAM,
+} NiceComponentProperty;
 
 static void
-component_schedule_io_callback (Component *component);
+nice_component_constructed (GObject *obj);
 static void
-component_deschedule_io_callback (Component *component);
+nice_component_get_property (GObject *obj,
+    guint property_id, GValue *value, GParamSpec *pspec);
+static void
+nice_component_set_property (GObject *obj,
+    guint property_id, const GValue *value, GParamSpec *pspec);
+static void
+nice_component_finalize (GObject *obj);
+
+static void
+nice_component_schedule_io_callback (NiceComponent *component);
+static void
+nice_component_deschedule_io_callback (NiceComponent *component);
+static void
+nice_component_detach_socket (NiceComponent *component, NiceSocket *nicesock);
+static void
+nice_component_clear_selected_pair (NiceComponent *component);
 
 
 void
@@ -74,12 +96,15 @@ incoming_check_free (IncomingCheck *icheck)
 }
 
 /* Must *not* take the agent lock, since it’s called from within
- * component_set_io_context(), which holds the Component’s I/O lock. */
+ * nice_component_set_io_context(), which holds the Component’s I/O lock. */
 static void
 socket_source_attach (SocketSource *socket_source, GMainContext *context)
 {
   GSource *source;
 
+  if (socket_source->socket->fileno == NULL)
+    return;
+
   /* Create a source. */
   source = g_socket_create_source (socket_source->socket->fileno,
       G_IO_IN, NULL);
@@ -121,50 +146,57 @@ socket_source_free (SocketSource *source)
   g_slice_free (SocketSource, source);
 }
 
-Component *
-component_new (guint id, NiceAgent *agent, Stream *stream)
+NiceComponent *
+nice_component_new (guint id, NiceAgent *agent, NiceStream *stream)
 {
-  Component *component;
-
-  g_atomic_int_inc (&n_components_created);
-  nice_debug ("Created NiceComponent (%u created, %u destroyed)",
-      n_components_created, n_components_destroyed);
-
-  component = g_slice_new0 (Component);
-  component->id = id;
-  component->state = NICE_COMPONENT_STATE_DISCONNECTED;
-  component->restart_candidate = NULL;
-  component->tcp = NULL;
-  component->agent = agent;
-  component->stream = stream;
-
-  nice_agent_init_stun_agent (agent, &component->stun_agent);
-
-  g_mutex_init (&component->io_mutex);
-  g_queue_init (&component->pending_io_messages);
-  component->io_callback_id = 0;
-
-  component->own_ctx = g_main_context_new ();
-  component->stop_cancellable = g_cancellable_new ();
-  component->stop_cancellable_source =
-      g_cancellable_source_new (component->stop_cancellable);
-  g_source_set_dummy_callback (component->stop_cancellable_source);
-  g_source_attach (component->stop_cancellable_source, component->own_ctx);
-  component->ctx = g_main_context_ref (component->own_ctx);
-
-  /* Start off with a fresh main context and all I/O paused. This
-   * will be updated when nice_agent_attach_recv() or nice_agent_recv_messages()
-   * are called. */
-  component_set_io_context (component, NULL);
-  component_set_io_callback (component, NULL, NULL, NULL, 0, NULL);
-
-  g_queue_init (&component->queued_tcp_packets);
-
-  return component;
+  return g_object_new (NICE_TYPE_COMPONENT,
+                       "id", id,
+                       "agent", agent,
+                       "stream", stream,
+                       NULL);
 }
 
 void
-component_clean_turn_servers (Component *cmp)
+nice_component_remove_socket (NiceComponent *cmp, NiceSocket *nsocket)
+{
+  GSList *i;
+
+  for (i = cmp->local_candidates; i;) {
+    NiceCandidate *candidate = i->data;
+    GSList *next = i->next;
+
+    if (!nice_socket_is_based_on (candidate->sockptr, nsocket)) {
+      i = next;
+      continue;
+    }
+
+    if (candidate == cmp->selected_pair.local) {
+      nice_component_clear_selected_pair (cmp);
+      agent_signal_component_state_change (cmp->agent, cmp->stream->id,
+          cmp->id, NICE_COMPONENT_STATE_FAILED);
+    }
+
+    refresh_prune_candidate (cmp->agent, candidate);
+    if (candidate->sockptr != nsocket) {
+      discovery_prune_socket (cmp->agent, candidate->sockptr);
+      conn_check_prune_socket (cmp->agent, cmp->stream, cmp,
+          candidate->sockptr);
+      nice_component_detach_socket (cmp, candidate->sockptr);
+    }
+    agent_remove_local_candidate (cmp->agent, candidate);
+    nice_candidate_free (candidate);
+
+    cmp->local_candidates = g_slist_delete_link (cmp->local_candidates, i);
+    i = next;
+  }
+
+  discovery_prune_socket (cmp->agent, nsocket);
+  conn_check_prune_socket (cmp->agent, cmp->stream, cmp, nsocket);
+  nice_component_detach_socket (cmp, nsocket);
+}
+
+void
+nice_component_clean_turn_servers (NiceComponent *cmp)
 {
   GSList *i;
 
@@ -195,7 +227,7 @@ component_clean_turn_servers (Component *cmp)
         discovery_prune_socket (cmp->agent, cmp->turn_candidate->sockptr);
         conn_check_prune_socket (cmp->agent, cmp->stream, cmp,
             cmp->turn_candidate->sockptr);
-        component_detach_socket (cmp, cmp->turn_candidate->sockptr);
+        nice_component_detach_socket (cmp, cmp->turn_candidate->sockptr);
 	nice_candidate_free (cmp->turn_candidate);
       }
       /* Bring the priority down to 0, so that it will be replaced
@@ -208,7 +240,7 @@ component_clean_turn_servers (Component *cmp)
       discovery_prune_socket (cmp->agent, candidate->sockptr);
       conn_check_prune_socket (cmp->agent, cmp->stream, cmp,
           candidate->sockptr);
-      component_detach_socket (cmp, candidate->sockptr);
+      nice_component_detach_socket (cmp, candidate->sockptr);
       agent_remove_local_candidate (cmp->agent, candidate);
       nice_candidate_free (candidate);
     }
@@ -218,7 +250,7 @@ component_clean_turn_servers (Component *cmp)
 }
 
 static void
-component_clear_selected_pair (Component *component)
+nice_component_clear_selected_pair (NiceComponent *component)
 {
   if (component->selected_pair.keepalive.tick_source != NULL) {
     g_source_destroy (component->selected_pair.keepalive.tick_source);
@@ -232,7 +264,7 @@ component_clear_selected_pair (Component *component)
 /* Must be called with the agent lock held as it touches internal Component
  * state. */
 void
-component_close (Component *cmp)
+nice_component_close (NiceComponent *cmp)
 {
   IOCallbackData *data;
   GOutputVector *vec;
@@ -240,13 +272,13 @@ component_close (Component *cmp)
   /* Start closing the pseudo-TCP socket first. FIXME: There is a very big and
    * reliably triggerable race here. pseudo_tcp_socket_close() does not block
    * on the socket closing — it only sends the first packet of the FIN
-   * handshake. component_close() will immediately afterwards close the
+   * handshake. nice_component_close() will immediately afterwards close the
    * underlying component sockets, aborting the handshake.
    *
    * On the principle that starting the FIN handshake is better than not
    * starting it, even if it’s later truncated, call pseudo_tcp_socket_close().
-   * A long-term fix is needed in the form of making component_close() (and all
-   * its callers) async, so we can properly block on closure. */
+   * A long-term fix is needed in the form of making nice_component_close() (and
+   * all its callers) async, so we can properly block on closure. */
   if (cmp->tcp) {
     pseudo_tcp_socket_close (cmp->tcp, TRUE);
   }
@@ -269,12 +301,12 @@ component_close (Component *cmp)
   g_slist_free_full (cmp->remote_candidates,
       (GDestroyNotify) nice_candidate_free);
   cmp->remote_candidates = NULL;
-  component_free_socket_sources (cmp);
+  nice_component_free_socket_sources (cmp);
   g_slist_free_full (cmp->incoming_checks,
       (GDestroyNotify) incoming_check_free);
   cmp->incoming_checks = NULL;
 
-  component_clean_turn_servers (cmp);
+  nice_component_clean_turn_servers (cmp);
 
   if (cmp->tcp_clock) {
     g_source_destroy (cmp->tcp_clock);
@@ -289,7 +321,7 @@ component_close (Component *cmp)
   while ((data = g_queue_pop_head (&cmp->pending_io_messages)) != NULL)
     io_callback_data_free (data);
 
-  component_deschedule_io_callback (cmp);
+  nice_component_deschedule_io_callback (cmp);
 
   g_cancellable_cancel (cmp->stop_cancellable);
 
@@ -299,47 +331,13 @@ component_close (Component *cmp)
   }
 }
 
-/* Must be called with the agent lock released as it could dispose of
- * NiceIOStreams. */
-void
-component_free (Component *cmp)
-{
-  /* Component should have been closed already. */
-  g_warn_if_fail (cmp->local_candidates == NULL);
-  g_warn_if_fail (cmp->remote_candidates == NULL);
-  g_warn_if_fail (cmp->incoming_checks == NULL);
-
-  g_clear_object (&cmp->tcp);
-  g_clear_object (&cmp->stop_cancellable);
-  g_clear_object (&cmp->iostream);
-  g_mutex_clear (&cmp->io_mutex);
-
-  if (cmp->stop_cancellable_source != NULL) {
-    g_source_destroy (cmp->stop_cancellable_source);
-    g_source_unref (cmp->stop_cancellable_source);
-  }
-
-  if (cmp->ctx != NULL) {
-    g_main_context_unref (cmp->ctx);
-    cmp->ctx = NULL;
-  }
-
-  g_main_context_unref (cmp->own_ctx);
-
-  g_slice_free (Component, cmp);
-
-  g_atomic_int_inc (&n_components_destroyed);
-  nice_debug ("Destroyed NiceComponent (%u created, %u destroyed)",
-      n_components_created, n_components_destroyed);
-}
-
 /*
  * Finds a candidate pair that has matching foundation ids.
  *
  * @return TRUE if pair found, pointer to pair stored at 'pair'
  */
 gboolean
-component_find_pair (Component *cmp, NiceAgent *agent, const gchar *lfoundation, const gchar *rfoundation, CandidatePair *pair)
+nice_component_find_pair (NiceComponent *cmp, NiceAgent *agent, const gchar *lfoundation, const gchar *rfoundation, CandidatePair *pair)
 {
   GSList *i;
   CandidatePair result = { 0, };
@@ -375,7 +373,7 @@ component_find_pair (Component *cmp, NiceAgent *agent, const gchar *lfoundation,
  * session.
  */
 void
-component_restart (Component *cmp)
+nice_component_restart (NiceComponent *cmp)
 {
   GSList *i;
 
@@ -410,7 +408,8 @@ component_restart (Component *cmp)
  * Changes the selected pair for the component to 'pair'. Does not
  * emit the "selected-pair-changed" signal.
  */ 
-void component_update_selected_pair (Component *component, const CandidatePair *pair)
+void
+nice_component_update_selected_pair (NiceComponent *component, const CandidatePair *pair)
 {
   g_assert (component);
   g_assert (pair);
@@ -425,17 +424,17 @@ void component_update_selected_pair (Component *component, const CandidatePair *
         component->turn_candidate->sockptr);
     conn_check_prune_socket (component->agent, component->stream, component,
         component->turn_candidate->sockptr);
-    component_detach_socket (component, component->turn_candidate->sockptr);
+    nice_component_detach_socket (component, component->turn_candidate->sockptr);
     nice_candidate_free (component->turn_candidate);
     component->turn_candidate = NULL;
   }
 
-  component_clear_selected_pair (component);
+  nice_component_clear_selected_pair (component);
 
   component->selected_pair.local = pair->local;
   component->selected_pair.remote = pair->remote;
   component->selected_pair.priority = pair->priority;
-
+  component->selected_pair.prflx_priority = pair->prflx_priority;
 }
 
 /*
@@ -445,7 +444,7 @@ void component_update_selected_pair (Component *component, const CandidatePair *
  * @return pointer to candidate or NULL if not found
  */
 NiceCandidate *
-component_find_remote_candidate (const Component *component, const NiceAddress *addr, NiceCandidateTransport transport)
+nice_component_find_remote_candidate (NiceComponent *component, const NiceAddress *addr, NiceCandidateTransport transport)
 {
   GSList *i;
 
@@ -469,8 +468,8 @@ component_find_remote_candidate (const Component *component, const NiceAddress *
  */
 
 NiceCandidate *
-component_set_selected_remote_candidate (NiceAgent *agent, Component *component,
-    NiceCandidate *candidate)
+nice_component_set_selected_remote_candidate (NiceComponent *component,
+    NiceAgent *agent, NiceCandidate *candidate)
 {
   NiceCandidate *local = NULL;
   NiceCandidate *remote = NULL;
@@ -483,7 +482,7 @@ component_set_selected_remote_candidate (NiceAgent *agent, Component *component,
     NiceCandidate *tmp = item->data;
     guint64 tmp_prio = 0;
 
-    if (tmp->transport != candidate->transport ||
+    if (tmp->transport != conn_check_match_transport(candidate->transport) ||
 	tmp->addr.s.addr.sa_family != candidate->addr.s.addr.sa_family ||
         tmp->type != NICE_CANDIDATE_TYPE_HOST)
       continue;
@@ -499,7 +498,7 @@ component_set_selected_remote_candidate (NiceAgent *agent, Component *component,
   if (local == NULL)
     return NULL;
 
-  remote = component_find_remote_candidate (component, &candidate->addr,
+  remote = nice_component_find_remote_candidate (component, &candidate->addr,
       candidate->transport);
 
   if (!remote) {
@@ -509,7 +508,7 @@ component_set_selected_remote_candidate (NiceAgent *agent, Component *component,
     agent_signal_new_remote_candidate (agent, remote);
   }
 
-  component_clear_selected_pair (component);
+  nice_component_clear_selected_pair (component);
 
   component->selected_pair.local = local;
   component->selected_pair.remote = remote;
@@ -530,7 +529,7 @@ _find_socket_source (gconstpointer a, gconstpointer b)
 /* This takes ownership of the socket.
  * It creates and attaches a source to the component’s context. */
 void
-component_attach_socket (Component *component, NiceSocket *nicesock)
+nice_component_attach_socket (NiceComponent *component, NiceSocket *nicesock)
 {
   GSList *l;
   SocketSource *socket_source;
@@ -540,9 +539,6 @@ component_attach_socket (Component *component, NiceSocket *nicesock)
 
   g_assert (component->ctx != NULL);
 
-  if (nicesock->fileno == NULL)
-    return;
-
   /* Find an existing SocketSource in the component which contains @socket, or
    * create a new one.
    *
@@ -559,7 +555,8 @@ component_attach_socket (Component *component, NiceSocket *nicesock)
     socket_source->component = component;
     component->socket_sources =
         g_slist_prepend (component->socket_sources, socket_source);
-    component->socket_sources_age++;
+    if (nicesock->fileno != NULL)
+      component->socket_sources_age++;
   }
 
   /* Create and attach a source */
@@ -571,9 +568,9 @@ component_attach_socket (Component *component, NiceSocket *nicesock)
 /* Reattaches socket handles of @component to the main context.
  *
  * Must *not* take the agent lock, since it’s called from within
- * component_set_io_context(), which holds the Component’s I/O lock. */
+ * nice_component_set_io_context(), which holds the Component’s I/O lock. */
 static void
-component_reattach_all_sockets (Component *component)
+nice_component_reattach_all_sockets (NiceComponent *component)
 {
   GSList *i;
 
@@ -586,8 +583,8 @@ component_reattach_all_sockets (Component *component)
 }
 
 /**
- * component_detach_socket:
- * @component: a #Component
+ * nice_component_detach_socket:
+ * @component: a #NiceComponent
  * @socket: the socket to detach the source for
  *
  * Detach the #GSource for the single specified @socket. It also closes it
@@ -595,8 +592,8 @@ component_reattach_all_sockets (Component *component)
  *
  * If the @socket doesn’t exist in this @component, do nothing.
  */
-void
-component_detach_socket (Component *component, NiceSocket *nicesock)
+static void
+nice_component_detach_socket (NiceComponent *component, NiceSocket *nicesock)
 {
   GSList *l;
   SocketSource *socket_source;
@@ -637,10 +634,10 @@ component_detach_socket (Component *component, NiceSocket *nicesock)
  * sockets themselves untouched.
  *
  * Must *not* take the agent lock, since it’s called from within
- * component_set_io_context(), which holds the Component’s I/O lock.
+ * nice_component_set_io_context(), which holds the Component’s I/O lock.
  */
 void
-component_detach_all_sockets (Component *component)
+nice_component_detach_all_sockets (NiceComponent *component)
 {
   GSList *i;
 
@@ -653,7 +650,7 @@ component_detach_all_sockets (Component *component)
 }
 
 void
-component_free_socket_sources (Component *component)
+nice_component_free_socket_sources (NiceComponent *component)
 {
   nice_debug ("Free socket sources for component %p.", component);
 
@@ -662,11 +659,11 @@ component_free_socket_sources (Component *component)
   component->socket_sources = NULL;
   component->socket_sources_age++;
 
-  component_clear_selected_pair (component);
+  nice_component_clear_selected_pair (component);
 }
 
 GMainContext *
-component_dup_io_context (Component *component)
+nice_component_dup_io_context (NiceComponent *component)
 {
   return g_main_context_ref (component->own_ctx);
 }
@@ -674,7 +671,7 @@ component_dup_io_context (Component *component)
 /* If @context is %NULL, it's own context is used, so component->ctx is always
  * guaranteed to be non-%NULL. */
 void
-component_set_io_context (Component *component, GMainContext *context)
+nice_component_set_io_context (NiceComponent *component, GMainContext *context)
 {
   g_mutex_lock (&component->io_mutex);
 
@@ -684,11 +681,11 @@ component_set_io_context (Component *component, GMainContext *context)
     else
       g_main_context_ref (context);
 
-    component_detach_all_sockets (component);
+    nice_component_detach_all_sockets (component);
     g_main_context_unref (component->ctx);
 
     component->ctx = context;
-    component_reattach_all_sockets (component);
+    nice_component_reattach_all_sockets (component);
   }
 
   g_mutex_unlock (&component->io_mutex);
@@ -705,7 +702,7 @@ component_set_io_context (Component *component, GMainContext *context)
  * emitted for it (which could cause data loss if the I/O callback function was
  * unset in that time). */
 void
-component_set_io_callback (Component *component,
+nice_component_set_io_callback (NiceComponent *component,
     NiceAgentRecvFunc func, gpointer user_data,
     NiceInputMessage *recv_messages, guint n_recv_messages,
     GError **error)
@@ -722,14 +719,14 @@ component_set_io_callback (Component *component,
     component->recv_messages = NULL;
     component->n_recv_messages = 0;
 
-    component_schedule_io_callback (component);
+    nice_component_schedule_io_callback (component);
   } else {
     component->io_callback = NULL;
     component->io_user_data = NULL;
     component->recv_messages = recv_messages;
     component->n_recv_messages = n_recv_messages;
 
-    component_deschedule_io_callback (component);
+    nice_component_deschedule_io_callback (component);
   }
 
   nice_input_message_iter_reset (&component->recv_messages_iter);
@@ -739,7 +736,7 @@ component_set_io_callback (Component *component,
 }
 
 gboolean
-component_has_io_callback (Component *component)
+nice_component_has_io_callback (NiceComponent *component)
 {
   gboolean has_io_callback;
 
@@ -775,7 +772,7 @@ io_callback_data_free (IOCallbackData *data)
 static gboolean
 emit_io_callback_cb (gpointer user_data)
 {
-  Component *component = user_data;
+  NiceComponent *component = user_data;
   IOCallbackData *data;
   NiceAgentRecvFunc io_callback;
   gpointer io_user_data;
@@ -792,7 +789,7 @@ emit_io_callback_cb (gpointer user_data)
   g_mutex_lock (&component->io_mutex);
 
   /* The members of Component are guaranteed not to have changed since this
-   * GSource was attached in component_emit_io_callback(). The Component’s agent
+   * GSource was attached in nice_component_emit_io_callback(). The Component’s agent
    * and stream are immutable after construction, as are the stream and
    * component IDs. The callback and its user data may have changed, but are
    * guaranteed to be non-%NULL at the start as the idle source is removed when
@@ -802,7 +799,7 @@ emit_io_callback_cb (gpointer user_data)
    *
    * If the component is destroyed (which happens if the agent or stream are
    * destroyed) between attaching the GSource and firing it, the GSource is
-   * detached in component_free() and this callback is never invoked. If the
+   * detached during dispose and this callback is never invoked. If the
    * agent is destroyed during an io_callback, its weak pointer will be
    * nullified. Similarly, the Component needs to be re-queried for after every
    * iteration, just in case the client has removed the stream in the
@@ -845,7 +842,7 @@ emit_io_callback_cb (gpointer user_data)
 
 /* This must be called with the agent lock *held*. */
 void
-component_emit_io_callback (Component *component,
+nice_component_emit_io_callback (NiceComponent *component,
     const guint8 *buf, gsize buf_len)
 {
   NiceAgent *agent;
@@ -897,7 +894,7 @@ component_emit_io_callback (Component *component,
 
     nice_debug ("%s: **WARNING: SLOW PATH**", G_STRFUNC);
 
-    component_schedule_io_callback (component);
+    nice_component_schedule_io_callback (component);
 
     g_mutex_unlock (&component->io_mutex);
   }
@@ -905,7 +902,7 @@ component_emit_io_callback (Component *component,
 
 /* Note: Must be called with the io_mutex held. */
 static void
-component_schedule_io_callback (Component *component)
+nice_component_schedule_io_callback (NiceComponent *component)
 {
   GSource *source;
 
@@ -928,7 +925,7 @@ component_schedule_io_callback (Component *component)
 
 /* Note: Must be called with the io_mutex held. */
 static void
-component_deschedule_io_callback (Component *component)
+nice_component_deschedule_io_callback (NiceComponent *component)
 {
   /* Already descheduled? */
   if (component->io_callback_id == 0)
@@ -938,6 +935,202 @@ component_deschedule_io_callback (Component *component)
   component->io_callback_id = 0;
 }
 
+static void
+nice_component_class_init (NiceComponentClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->constructed = nice_component_constructed;
+  object_class->get_property = nice_component_get_property;
+  object_class->set_property = nice_component_set_property;
+  object_class->finalize = nice_component_finalize;
+
+  /**
+   * NiceComponent:id:
+   *
+   * The unique numeric ID of the component.
+   *
+   * Since: 0.1.14
+   */
+  g_object_class_install_property (object_class, PROP_ID,
+      g_param_spec_uint (
+         "id",
+         "ID",
+         "The unique numeric ID of the component.",
+         1, G_MAXUINT, 1,
+         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+  /**
+   * NiceComponent:agent:
+   *
+   * The #NiceAgent this component belongs to.
+   *
+   * Since: 0.1.14
+   */
+  g_object_class_install_property (object_class, PROP_AGENT,
+      g_param_spec_object (
+         "agent",
+         "Agent",
+         "The NiceAgent this component belongs to.",
+         NICE_TYPE_AGENT,
+         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+  /**
+   * NiceComponent:stream:
+   *
+   * The #NiceStream this component belongs to.
+   *
+   * Since: 0.1.14
+   */
+  g_object_class_install_property (object_class, PROP_STREAM,
+      g_param_spec_object (
+         "stream",
+         "Stream",
+         "The NiceStream this component belongs to.",
+         NICE_TYPE_STREAM,
+         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+}
+
+static void
+nice_component_init (NiceComponent *component)
+{
+  g_atomic_int_inc (&n_components_created);
+  nice_debug ("Created NiceComponent (%u created, %u destroyed)",
+      n_components_created, n_components_destroyed);
+
+  component->id = 0;
+  component->state = NICE_COMPONENT_STATE_DISCONNECTED;
+  component->restart_candidate = NULL;
+  component->tcp = NULL;
+  component->agent = NULL;
+  component->stream = NULL;
+
+  g_mutex_init (&component->io_mutex);
+  g_queue_init (&component->pending_io_messages);
+  component->io_callback_id = 0;
+
+  component->own_ctx = g_main_context_new ();
+  component->stop_cancellable = g_cancellable_new ();
+  component->stop_cancellable_source =
+      g_cancellable_source_new (component->stop_cancellable);
+  g_source_set_dummy_callback (component->stop_cancellable_source);
+  g_source_attach (component->stop_cancellable_source, component->own_ctx);
+  component->ctx = g_main_context_ref (component->own_ctx);
+
+  /* Start off with a fresh main context and all I/O paused. This
+   * will be updated when nice_agent_attach_recv() or nice_agent_recv_messages()
+   * are called. */
+  nice_component_set_io_context (component, NULL);
+  nice_component_set_io_callback (component, NULL, NULL, NULL, 0, NULL);
+
+  g_queue_init (&component->queued_tcp_packets);
+}
+
+static void
+nice_component_constructed (GObject *obj)
+{
+  NiceComponent *component;
+
+  component = NICE_COMPONENT (obj);
+
+  g_assert (component->agent != NULL);
+  nice_agent_init_stun_agent (component->agent, &component->stun_agent);
+
+  G_OBJECT_CLASS (nice_component_parent_class)->constructed (obj);
+}
+
+static void
+nice_component_get_property (GObject *obj,
+    guint property_id, GValue *value, GParamSpec *pspec)
+{
+  NiceComponent *component;
+
+  component = NICE_COMPONENT (obj);
+
+  switch ((NiceComponentProperty) property_id)
+    {
+    case PROP_ID:
+      g_value_set_uint (value, component->id);
+      break;
+
+    case PROP_AGENT:
+      g_value_set_object (value, component->agent);
+      break;
+
+    case PROP_STREAM:
+      g_value_set_object (value, component->stream);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, property_id, pspec);
+    }
+}
+
+static void
+nice_component_set_property (GObject *obj,
+    guint property_id, const GValue *value, GParamSpec *pspec)
+{
+  NiceComponent *component;
+
+  component = NICE_COMPONENT (obj);
+
+  switch ((NiceComponentProperty) property_id)
+    {
+    case PROP_ID:
+      component->id = g_value_get_uint (value);
+      break;
+
+    case PROP_AGENT:
+      component->agent = g_value_get_object (value);
+      break;
+
+    case PROP_STREAM:
+      component->stream = g_value_get_object (value);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, property_id, pspec);
+    }
+}
+
+/* Must be called with the agent lock released as it could dispose of
+ * NiceIOStreams. */
+static void
+nice_component_finalize (GObject *obj)
+{
+  NiceComponent *cmp;
+
+  cmp = NICE_COMPONENT (obj);
+
+  /* Component should have been closed already. */
+  g_warn_if_fail (cmp->local_candidates == NULL);
+  g_warn_if_fail (cmp->remote_candidates == NULL);
+  g_warn_if_fail (cmp->incoming_checks == NULL);
+
+  g_clear_object (&cmp->tcp);
+  g_clear_object (&cmp->stop_cancellable);
+  g_clear_object (&cmp->iostream);
+  g_mutex_clear (&cmp->io_mutex);
+
+  if (cmp->stop_cancellable_source != NULL) {
+    g_source_destroy (cmp->stop_cancellable_source);
+    g_source_unref (cmp->stop_cancellable_source);
+  }
+
+  if (cmp->ctx != NULL) {
+    g_main_context_unref (cmp->ctx);
+    cmp->ctx = NULL;
+  }
+
+  g_main_context_unref (cmp->own_ctx);
+
+  g_atomic_int_inc (&n_components_destroyed);
+  nice_debug ("Destroyed NiceComponent (%u created, %u destroyed)",
+      n_components_created, n_components_destroyed);
+
+  G_OBJECT_CLASS (nice_component_parent_class)->finalize (obj);
+}
+
 /**
  * ComponentSource:
  *
@@ -980,7 +1173,7 @@ component_source_prepare (GSource *source, gint *timeout_)
 {
   ComponentSource *component_source = (ComponentSource *) source;
   NiceAgent *agent;
-  Component *component;
+  NiceComponent *component;
   GSList *parentl, *childl;
 
   agent = g_weak_ref_get (&component_source->agent_ref);
@@ -1011,6 +1204,9 @@ component_source_prepare (GSource *source, gint *timeout_)
     SocketSource *parent_socket_source = parentl->data;
     SocketSource *child_socket_source;
 
+    if (parent_socket_source->socket->fileno == NULL)
+      continue;
+
     /* Iterating the list of socket sources every time isn't a big problem
      * because the number of pairs is limited ~100 normally, so there will
      * rarely be more than 10.
@@ -1128,7 +1324,7 @@ static GSourceFuncs component_source_funcs = {
 };
 
 /**
- * component_source_new:
+ * nice_component_source_new:
  * @agent: a #NiceAgent
  * @stream_id: The stream's id
  * @component_id: The component's number
@@ -1150,7 +1346,7 @@ static GSourceFuncs component_source_funcs = {
  * Returns: (transfer full): a new #ComponentSource; unref with g_source_unref()
  */
 GSource *
-component_input_source_new (NiceAgent *agent, guint stream_id,
+nice_component_input_source_new (NiceAgent *agent, guint stream_id,
     guint component_id, GPollableInputStream *pollable_istream,
     GCancellable *cancellable)
 {
