@@ -50,8 +50,8 @@
  * component triple, and will be closed as soon as that stream is removed from
  * the agent (e.g. if nice_agent_remove_stream() is called from another thread).
  * If g_output_stream_close() is called on a #NiceOutputStream, the output
- * stream will be marked as closed, but the underlying #NiceAgent stream will
- * not be removed. Use nice_agent_remove_stream() to do that.
+ * stream and underlying #NiceAgent stream will be closed, but the underlying
+ * stream will not be removed. Use nice_agent_remove_stream() to do that.
  *
  * The output stream can only be used once the
  * #NiceAgent::reliable-transport-writable signal has been received for the
@@ -104,6 +104,8 @@ static void nice_output_stream_set_property (GObject *object, guint prop_id,
 
 static gssize nice_output_stream_write (GOutputStream *stream,
     const void *buffer, gsize count, GCancellable *cancellable, GError **error);
+static gboolean nice_output_stream_close (GOutputStream *stream,
+    GCancellable *cancellable, GError **error);
 
 static gboolean nice_output_stream_is_writable (GPollableOutputStream *stream);
 static gssize nice_output_stream_write_nonblocking (
@@ -122,6 +124,7 @@ nice_output_stream_class_init (NiceOutputStreamClass *klass)
   g_type_class_add_private (klass, sizeof (NiceOutputStreamPrivate));
 
   stream_class->write_fn = nice_output_stream_write;
+  stream_class->close_fn = nice_output_stream_close;
 
   gobject_class->set_property = nice_output_stream_set_property;
   gobject_class->get_property = nice_output_stream_get_property;
@@ -185,6 +188,11 @@ nice_output_stream_dispose (GObject *object)
   NiceOutputStream *self = NICE_OUTPUT_STREAM (object);
   NiceAgent *agent;
 
+  /* Ensure the stream is closed first, otherwise the agent can’t be found in
+   * the close handler called by the parent implementation. */
+  if (!g_output_stream_is_closed (G_OUTPUT_STREAM (object)))
+    g_output_stream_close (G_OUTPUT_STREAM (object), NULL, NULL);
+
   agent = g_weak_ref_get (&self->priv->agent_ref);
   if (agent != NULL) {
     g_signal_handlers_disconnect_by_func (agent, streams_removed_cb, self);
@@ -463,14 +471,43 @@ nice_output_stream_write (GOutputStream *stream, const void *buffer, gsize count
   return len;
 }
 
+static gboolean
+nice_output_stream_close (GOutputStream *stream, GCancellable *cancellable,
+    GError **error)
+{
+  NiceOutputStreamPrivate *priv = NICE_OUTPUT_STREAM (stream)->priv;
+  NiceComponent *component = NULL;
+  NiceStream *_stream = NULL;
+  NiceAgent *agent;  /* owned */
+
+  /* Has the agent disappeared? */
+  agent = g_weak_ref_get (&priv->agent_ref);
+  if (agent == NULL)
+    return TRUE;
+
+  agent_lock ();
+
+  /* Shut down the write side of the pseudo-TCP stream. */
+  if (agent_find_component (agent, priv->stream_id, priv->component_id,
+          &_stream, &component) && agent->reliable &&
+      !pseudo_tcp_socket_is_closed (component->tcp)) {
+    pseudo_tcp_socket_shutdown (component->tcp, PSEUDO_TCP_SHUTDOWN_WR);
+  }
+
+  agent_unlock ();
+
+  g_object_unref (agent);
+
+  return TRUE;
+}
+
 static gboolean
 nice_output_stream_is_writable (GPollableOutputStream *stream)
 {
   NiceOutputStreamPrivate *priv = NICE_OUTPUT_STREAM (stream)->priv;
-  Component *component = NULL;
-  Stream *_stream = NULL;
+  NiceComponent *component = NULL;
+  NiceStream *_stream = NULL;
   gboolean retval = FALSE;
-  GSList *i;
   NiceAgent *agent;  /* owned */
 
   /* Closed streams are not writeable. */
@@ -490,22 +527,15 @@ nice_output_stream_is_writable (GPollableOutputStream *stream)
         priv->stream_id);
     goto done;
   }
+  if (component->selected_pair.local != NULL) {
+    NiceSocket *sockptr = component->selected_pair.local->sockptr;
 
-  /* If it’s a reliable agent, see if there’s any space in the pseudo-TCP output
-   * buffer. */
-  if (component->tcp != NULL) {
-    retval = pseudo_tcp_socket_can_send (component->tcp);
-    goto done;
-  }
-
-  /* Check whether any of the component’s FDs are pollable. */
-  for (i = component->socket_sources; i != NULL; i = i->next) {
-    SocketSource *socket_source = i->data;
-    NiceSocket *nicesock = socket_source->socket;
-
-    if (g_socket_condition_check (nicesock->fileno, G_IO_OUT) != 0) {
-      retval = TRUE;
-      break;
+    /* If it’s a reliable agent, see if there’s any space in the pseudo-TCP
+     * output buffer. */
+    if (!nice_socket_is_reliable (sockptr)) {
+      retval = pseudo_tcp_socket_can_send (component->tcp);
+    } else {
+      retval = (g_socket_condition_check (sockptr->fileno, G_IO_OUT) != 0);
     }
   }
 
@@ -540,14 +570,9 @@ nice_output_stream_write_nonblocking (GPollableOutputStream *stream,
     return -1;
   }
 
-  if (count == 0)
-    return 0;
-
-  /* This is equivalent to the default GPollableOutputStream implementation. */
-  if (!g_pollable_output_stream_is_writable (stream)) {
-    g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK,
-        g_strerror (EAGAIN));
-    return -1;
+  if (count == 0) {
+    n_sent = 0;
+    goto done;
   }
 
   n_sent = nice_agent_send (agent, priv->stream_id, priv->component_id,
@@ -557,7 +582,7 @@ nice_output_stream_write_nonblocking (GPollableOutputStream *stream,
     g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK,
         g_strerror (EAGAIN));
 
-
+ done:
 
   g_object_unref (agent);
 
@@ -570,8 +595,8 @@ nice_output_stream_create_source (GPollableOutputStream *stream,
 {
   NiceOutputStreamPrivate *priv = NICE_OUTPUT_STREAM (stream)->priv;
   GSource *component_source = NULL;
-  Component *component = NULL;
-  Stream *_stream = NULL;
+  NiceComponent *component = NULL;
+  NiceStream *_stream = NULL;
   NiceAgent *agent;  /* owned */
 
   component_source = g_pollable_source_new (G_OBJECT (stream));
