@@ -42,7 +42,7 @@
 
 #include <glib.h>
 
-typedef struct _Stream Stream;
+typedef struct _NiceStream NiceStream;
 
 #include "component.h"
 #include "random.h"
@@ -59,13 +59,27 @@ G_BEGIN_DECLS
 #define NICE_STREAM_DEF_UFRAG   4 + 1    /* ufrag + NULL */
 #define NICE_STREAM_DEF_PWD     22 + 1   /* pwd + NULL */
 
-struct _Stream
-{
+#define NICE_TYPE_STREAM nice_stream_get_type()
+#define NICE_STREAM(obj) \
+  (G_TYPE_CHECK_INSTANCE_CAST ((obj), NICE_TYPE_STREAM, NiceStream))
+#define NICE_STREAM_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST ((klass), NICE_TYPE_STREAM, NiceStreamClass))
+#define NICE_IS_STREAM(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NICE_TYPE_STREAM))
+#define NICE_IS_STREAM_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_TYPE ((klass), NICE_TYPE_STREAM))
+#define NICE_STREAM_GET_CLASS(obj) \
+  (G_TYPE_INSTANCE_GET_CLASS ((obj), NICE_TYPE_STREAM, NiceStreamClass))
+
+struct _NiceStream {
+  /*< private >*/
+  GObject parent;
+
   gchar *name;
   guint id;
   guint n_components;
   gboolean initial_binding_request_received;
-  GSList *components; /* list of 'Component' structs */
+  GSList *components; /* list of 'NiceComponent' objects */
   GSList *conncheck_list;         /* list of CandidateCheckPair items */
   gchar local_ufrag[NICE_STREAM_MAX_UFRAG];
   gchar local_password[NICE_STREAM_MAX_PWD];
@@ -74,29 +88,32 @@ struct _Stream
   gboolean gathering;
   gboolean gathering_started;
   gint tos;
+  guint tick_counter;
 };
 
+typedef struct {
+  GObjectClass parent_class;
+} NiceStreamClass;
 
-Stream *
-stream_new (guint n_components, NiceAgent *agent);
+GType nice_stream_get_type (void);
+
+NiceStream *
+nice_stream_new (guint n_components, NiceAgent *agent);
 
 void
-stream_close (Stream *stream);
-
-void
-stream_free (Stream *stream);
+nice_stream_close (NiceStream *stream);
 
 gboolean
-stream_all_components_ready (const Stream *stream);
+nice_stream_all_components_ready (NiceStream *stream);
 
-Component *
-stream_find_component_by_id (const Stream *stream, guint id);
+NiceComponent *
+nice_stream_find_component_by_id (NiceStream *stream, guint id);
 
 void
-stream_initialize_credentials (Stream *stream, NiceRNG *rng);
+nice_stream_initialize_credentials (NiceStream *stream, NiceRNG *rng);
 
 void
-stream_restart (NiceAgent *agent, Stream *stream);
+nice_stream_restart (NiceStream *stream, NiceAgent *agent);
 
 G_END_DECLS
 
