@@ -41,10 +41,13 @@
 
 G_BEGIN_DECLS
 
+NiceSocket *
+nice_tcp_bsd_socket_new (GMainContext *ctx, NiceAddress *remote_addr,
+    NiceAddress *local_addr, gboolean reliable);
 
 NiceSocket *
-nice_tcp_bsd_socket_new (GMainContext *ctx, NiceAddress *addr);
-
+nice_tcp_bsd_socket_new_from_gsock (GMainContext *ctx, GSocket *gsock,
+    NiceAddress *remote_addr, NiceAddress *local_addr, gboolean reliable);
 
 G_END_DECLS
 
