@@ -427,14 +427,22 @@ stun_message_append_addr (StunMessage *msg, StunAttribute type,
   uint16_t alen, port;
   uint8_t family;
 
+  union {
+    const struct sockaddr *addr;
+    const struct sockaddr_in *in;
+    const struct sockaddr_in6 *in6;
+  } sa;
+
   if ((size_t) addrlen < sizeof (struct sockaddr))
     return STUN_MESSAGE_RETURN_INVALID;
 
+  sa.addr = addr;
+
   switch (addr->sa_family)
   {
     case AF_INET:
       {
-        const struct sockaddr_in *ip4 = (const struct sockaddr_in *)addr;
+        const struct sockaddr_in *ip4 = sa.in;
         family = 1;
         port = ip4->sin_port;
         alen = 4;
@@ -444,7 +452,7 @@ stun_message_append_addr (StunMessage *msg, StunAttribute type,
 
     case AF_INET6:
       {
-        const struct sockaddr_in6 *ip6 = (const struct sockaddr_in6 *)addr;
+        const struct sockaddr_in6 *ip6 = sa.in6;
         if ((size_t) addrlen < sizeof (*ip6))
           return STUN_MESSAGE_RETURN_INVALID;
 
@@ -538,25 +546,24 @@ stun_message_append_error (StunMessage *msg, StunError code)
  * they’re STUN packets. If they look like they might be, their buffers are
  * compacted to allow a more thorough check. */
 ssize_t stun_message_validate_buffer_length_fast (StunInputVector *buffers,
-    unsigned int n_buffers, size_t total_length, bool has_padding)
+    int n_buffers, size_t total_length, bool has_padding)
 {
   size_t mlen;
 
-  if (total_length < 1 || n_buffers < 1)
+  if (total_length < 1 || n_buffers == 0 || buffers[0].buffer == NULL)
   {
-    stun_debug ("STUN error: No data!\n");
+    stun_debug ("STUN error: No data!");
     return STUN_MESSAGE_BUFFER_INVALID;
   }
 
   if (buffers[0].buffer[0] >> 6)
   {
-    stun_debug ("STUN error: RTP or other non-protocol packet!\n");
     return STUN_MESSAGE_BUFFER_INVALID; // RTP or other non-STUN packet
   }
 
   if (total_length < STUN_MESSAGE_LENGTH_POS + STUN_MESSAGE_LENGTH_LEN)
   {
-    stun_debug ("STUN error: Incomplete STUN message header!\n");
+    stun_debug ("STUN error: Incomplete STUN message header!");
     return STUN_MESSAGE_BUFFER_INCOMPLETE;
   }
 
@@ -569,7 +576,8 @@ ssize_t stun_message_validate_buffer_length_fast (StunInputVector *buffers,
     unsigned int i;
 
     /* Skip bytes. */
-    for (i = 0; i < n_buffers; i++) {
+    for (i = 0; (n_buffers >= 0 && i < (unsigned int) n_buffers) ||
+             (n_buffers < 0 && buffers[i].buffer != NULL); i++) {
       if (buffers[i].size <= skip_remaining)
         skip_remaining -= buffers[i].size;
       else
@@ -589,12 +597,12 @@ ssize_t stun_message_validate_buffer_length_fast (StunInputVector *buffers,
   mlen += STUN_MESSAGE_HEADER_LENGTH;
 
   if (has_padding && stun_padding (mlen)) {
-    stun_debug ("STUN error: Invalid message length: %u!\n", (unsigned)mlen);
+    stun_debug ("STUN error: Invalid message length: %u!", (unsigned)mlen);
     return STUN_MESSAGE_BUFFER_INVALID; // wrong padding
   }
 
   if (total_length < mlen) {
-    stun_debug ("STUN error: Incomplete message: %u of %u bytes!\n",
+    stun_debug ("STUN error: Incomplete message: %u of %u bytes!",
         (unsigned) total_length, (unsigned) mlen);
     return STUN_MESSAGE_BUFFER_INCOMPLETE; // partial message
   }
@@ -630,7 +638,7 @@ int stun_message_validate_buffer_length (const uint8_t *msg, size_t length,
     if (len < 4)
     {
       stun_debug ("STUN error: Incomplete STUN attribute header of length "
-          "%u bytes!\n", (unsigned)len);
+          "%u bytes!", (unsigned)len);
       return STUN_MESSAGE_BUFFER_INVALID;
     }
 
@@ -644,7 +652,7 @@ int stun_message_validate_buffer_length (const uint8_t *msg, size_t length,
 
     if (len < alen)
     {
-      stun_debug ("STUN error: %u instead of %u bytes for attribute!\n",
+      stun_debug ("STUN error: %u instead of %u bytes for attribute!",
           (unsigned)len, (unsigned)alen);
       return STUN_MESSAGE_BUFFER_INVALID; // no room for attribute value + padding
     }
