@@ -54,10 +54,6 @@ main (void)
   WSAStartup(0x0202, &w);
 #endif
   nice_address_init (&addr);
-  g_type_init ();
-#if !GLIB_CHECK_VERSION(2,31,8)
-  g_thread_init (NULL);
-#endif
 
   if (!nice_address_set_from_string (&addr, "127.0.0.1"))
     g_assert_not_reached ();
