@@ -440,8 +440,6 @@ main (void)
   WSAStartup (0x0202, &w);
 #endif
   nice_address_init (&addr);
-  g_type_init ();
-  g_thread_init (NULL);
 
   g_assert (nice_address_set_from_string (&addr, "127.0.0.1"));
 
