@@ -72,8 +72,6 @@ main (void)
   guint stream;
 
   nice_address_init (&addr);
-  g_type_init ();
-  g_thread_init(NULL);
 
   loop = g_main_loop_new (NULL, FALSE);
 
@@ -102,7 +100,9 @@ main (void)
 
   g_main_loop_run (loop);
 
+  nice_agent_remove_stream (agent, stream);
   g_object_unref (agent);
+
   return 0;
 }
 
